# TODO:
# - Perform series of tests to make compute_w robust

# - Add possibility to compute from pandas.DataFrame. Several choices:
#       - Separate function that allows a DataFrame
#       - Force DataFrame in args[0] of compute_w
#       - Check type of args[0] of compute_w and convert DataFrame to matrix
#       - Check type of args[0] of compute_w and write additionnal code adapted to pandas.DataFrame

# - Check that items are RANKED for each annotator:
#       -> Exception or Warning ?

import pandas as pd
import warnings

def compute_w(data):
    """ Computes kendall's W from a list of rating lists.
    0 indicates no agreement and 1 indicates unanimous agreement.
    
    Parameters
    ---------

    data : list
        List of lists with shape (n_items * n_annotators)

    Return
    ---------

    W : float
        Kendall's W [0:1]
    
    Example
    ---------

    annotations = [
        [1, 1, 1, 2], # item 1
        [2, 2, 2, 3], # item 2
        [3, 3, 3, 1], # item 3
    ]
    # Annotator #4 disagrees with the other annotators
    # Annotators #1, #2, #3 agree

    W = kendall_w(annotations)
    # output: 0.4375
    """
    
    assert isinstance(data, list), "You must pass a python list, {} found".format(type(data))
    assert all(isinstance(x, list) for x in data), "You must pass a list of python lists as input." # To test
    assert all(isinstance(x[y], int) for x in data for y in range(len(x))), "You must pass a list of lists of integers." # To test

    # Number of annotators
    m = len(data[0])
    # Tests
    if not all(len(i) == m for i in data):
        raise ValueError("Items must all have the same number of annotators. At least one sublist of argument 'data' has different length than the first sublist.")
    if m <= 1:
        raise ValueError("Kendall's W is irrevelent for only one annotator, try adding more lists to argument 'data'.")
    if m == 2:
        warnings.warn("Kendall's W is adapted to measure agreement between more than two annotators. The results might not be reliable in this case.")
    
    # Number of items
    n = len(data)
    # Tests
    if n <= 1:
        raise ValueError("Kendall's W is irrevelent for only one item, try adding more sublists to argument 'data'.")

    # Sum of each item ranks
    sums = [sum(x) for x in data]
    # Mean of ranking sums
    Rbar = sum(sums) / n
    # Sum of squared deviations from the mean
    S = sum([(sums[x] - Rbar) ** 2 for x in range(n)])

    W = (12 * S) / (m ** 2 * (n ** 3 - n))

    return W